/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.sql.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.*;

import com.intellij.openapi.project.Project;

import com.kiwisoft.db.*;
import com.kiwisoft.db.driver.DatabaseDriver;
import com.kiwisoft.db.export.*;
import com.kiwisoft.db.sql.*;
import com.kiwisoft.sqlPlugin.config.ExportConfiguration;
import com.kiwisoft.sqlPlugin.config.SQLPluginAppConfig;
import com.kiwisoft.sqlPlugin.settings.EditTemplateDialogStub;
import com.kiwisoft.sqlPlugin.templates.TemplateManager;
import com.kiwisoft.sqlPlugin.actions.ConfigureExportAction;
import com.kiwisoft.utils.DocumentAdapter;
import com.kiwisoft.utils.NotifyObject;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.gui.ConfirmationDialogStub;
import com.kiwisoft.utils.gui.IconManager;
import com.kiwisoft.utils.gui.text.SourceTextPane;
import com.kiwisoft.utils.gui.text.SyntaxDefinitionFactory;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.8 $, $Date: 2005/08/10 18:54:14 $
 */
public class QueryPanel extends JPanel
{
	public static final int ERROR=1;
	public static final int INFO=2;
	public static final int DEFAULT=0;

	private Project project;
	private Query query;

	private boolean errors;
	private JTextComponent tfQuery;
	private JTextPane tfResult;
	private JTabbedPane pnlResults;
	private StatusBar pnlStatus;
	private JButton btnAutoCommit;

	private QueryDocumentListener queryDocumentListener;
	private ConfigurationListener configurationListener;
	private DatabaseObserver databaseObserver;
	private Database currentDatabase;
	private TextPaneMouseListener textPaneListener;

	private ExecuteAction executeAction;
	private ExecuteAction executeComplexAction;
	private SuspendAction suspendAction;
	private AutoCommitAction autoCommitAction;

	private ExecutionThread executionThread;

	public QueryPanel(Project project, Query query)
	{
		assert query!=null;
		this.project=project;
		this.query=query;
		setLayout(new BorderLayout());

		executeAction=new ExecuteAction(false);
		executeComplexAction=new ExecuteAction(true);
		suspendAction=new SuspendAction();
		autoCommitAction=new AutoCommitAction();

		tfQuery=new SourceTextPane(SyntaxDefinitionFactory.getInstance().getSyntaxDefinition("sql"));

		if (query.getStatement()!=null) tfQuery.setText(query.getStatement());
		queryDocumentListener=new QueryDocumentListener();
		tfQuery.getDocument().addDocumentListener(queryDocumentListener);
		tfQuery.setEditable(!query.isSystemQuery());

		JScrollPane pnlQuery=new JScrollPane(tfQuery);

		JPanel pnlButtons=new JPanel(new GridBagLayout());
		pnlButtons.add(Utils.createButton(executeAction),
				new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
		pnlButtons.add(Utils.createButton(executeComplexAction),
				new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
		pnlButtons.add(Utils.createButton(suspendAction),
				new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
		pnlButtons.add(Utils.createButton(new StartExportAction()),
				new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
		pnlButtons.add(Utils.createButton(new CancelAction()),
				new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
		pnlButtons.add(Box.createGlue(),
				new GridBagConstraints(0, 6, 1, 1, 0.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));

		tfResult=new JTextPane();
		Style style=tfResult.addStyle("error", null);
		style.addAttribute(StyleConstants.ColorConstants.Foreground, Color.RED);
		style=tfResult.addStyle("info", null);
		style.addAttribute(StyleConstants.ColorConstants.Foreground, Color.BLUE);
		style=tfResult.addStyle("default", null);
		style.addAttribute(StyleConstants.ColorConstants.Foreground, Color.BLACK);

		pnlResults=new JTabbedPane();
		pnlResults.setTabPlacement(JTabbedPane.BOTTOM);
		pnlResults.addTab(null, IconManager.getIcon(getClass().getClassLoader(), IconNames.OUTPUT_MESSAGES), new JScrollPane(tfResult));
		pnlResults.setToolTipTextAt(0, "<html>Messages</html>");
		pnlResults.setPreferredSize(new Dimension(200, 200));

		btnAutoCommit=new JButton(autoCommitAction);
		btnAutoCommit.setBorder(new BevelBorder(BevelBorder.LOWERED));

		pnlStatus=new StatusBar();
		pnlStatus.setPreferredSize(new Dimension(20, 20));
		pnlStatus.addComponent(btnAutoCommit);

		JSplitPane splitPane=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setOneTouchExpandable(true);
		splitPane.setTopComponent(pnlQuery);
		splitPane.setBottomComponent(pnlResults);
		splitPane.setDividerLocation(100);

		JPanel pnlMain=new JPanel(new BorderLayout());
		pnlMain.add(splitPane, BorderLayout.CENTER);
		pnlMain.add(pnlStatus, BorderLayout.SOUTH);

		add(pnlButtons, BorderLayout.WEST);
		add(pnlMain, BorderLayout.CENTER);
	}

	public void addNotify()
	{
		super.addNotify();
		configurationListener=new ConfigurationListener();
		CommonConfiguration.getInstance(project).addObserver(configurationListener);
		databaseObserver=new DatabaseObserver();
		DatabaseManager databaseManager=DatabaseManager.getInstance(project);
		databaseManager.addObserver(databaseObserver);
		currentDatabase=databaseManager.getCurrentDatabase();
		if (currentDatabase!=null) currentDatabase.addObserver(databaseObserver);
		textPaneListener=new TextPaneMouseListener();
		tfQuery.addMouseListener(textPaneListener);
		updateActions();
	}

	public void removeNotify()
	{
		if (configurationListener!=null)
			CommonConfiguration.getInstance(project).deleteObserver(configurationListener);
		if (databaseObserver!=null)
		{
			if (currentDatabase!=null) currentDatabase.deleteObserver(databaseObserver);
			DatabaseManager.getInstance(project).deleteObserver(databaseObserver);
		}
		if (textPaneListener!=null) tfQuery.removeMouseListener(textPaneListener);
		super.removeNotify();
	}

	public Query getQuery()
	{
		return query;
	}

	public String getQueryText()
	{
		String selectedText=tfQuery.getSelectedText();
		if (!StringUtils.isEmpty(selectedText)) return selectedText;
		else return tfQuery.getText();
	}

	private void updateActions()
	{
		Database currentDatabase=DatabaseManager.getInstance(project).getCurrentDatabase();
		if (executeAction!=null) executeAction.setEnabled(currentDatabase!=null && !StringUtils.isEmpty(tfQuery.getText()));
		if (executeComplexAction!=null) executeComplexAction.setEnabled(currentDatabase!=null && !StringUtils.isEmpty(tfQuery.getText()));
		if (autoCommitAction!=null) autoCommitAction.update(currentDatabase);
		if (btnAutoCommit!=null && currentDatabase!=null)
		{
			Object value=currentDatabase.getProperty(DatabaseDriver.AUTO_COMMIT);
			if (Boolean.TRUE.equals(value)) btnAutoCommit.setForeground(Color.RED.darker());
			else if (Boolean.FALSE.equals(value)) btnAutoCommit.setForeground(Color.GREEN.darker());
			else btnAutoCommit.setForeground(Color.ORANGE.darker());
		}
	}

	private class QueryDocumentListener extends DocumentAdapter
	{
		public void changedUpdate(DocumentEvent e)
		{
			query.setStatement(tfQuery.getText());
			updateActions();
		}
	}

	private class DatabaseObserver implements Observer
	{
		public void update(Observable o, Object arg)
		{
			NotifyObject note=(NotifyObject)arg;
			if (o instanceof DatabaseManager)
			{
				if ("current database changed".equals(note.getArgument(0)))
				{
					if (currentDatabase!=null) currentDatabase.deleteObserver(this);
					currentDatabase=DatabaseManager.getInstance(project).getCurrentDatabase();
					if (currentDatabase!=null) currentDatabase.addObserver(this);
					updateActions();
				}
			}
			else if (o instanceof Database)
			{
				if (note.getArgument(0).equals(DatabaseDriver.AUTO_COMMIT.getId()+" changed"))
				{
					updateActions();
				}
			}
		}
	}

	protected void executeStatement(boolean complex)
	{
		if (executionThread==null || !executionThread.isAlive())
		{
			pnlStatus.showMessage("");

			if (!query.isSystemQuery())
			{
				String fullText=tfQuery.getText();
				String partialText=tfQuery.getSelectedText();

				String queryText=partialText;

				if (StringUtils.isEmpty(queryText))
					queryText=fullText;
				else
				{
					CommonConfiguration commonConfiguration=CommonConfiguration.getInstance(project);
					Boolean confirm=commonConfiguration.getConfirmation(CommonConfiguration.PARTIAL_EXECUTE);
					if (Boolean.FALSE.equals(confirm))
					{
						queryText=fullText;
					}
					else if (confirm==null)
					{
						ConfirmationDialogStub dialog=new ConfirmationDialogStub("Question", "Do you want to execute only the selected part of the query?",
								new String[]{"Yes", "No", "Cancel"}, "Yes", true);
						Utils.wrapDialog(project, dialog, true, true);
						if ("Yes".equals(dialog.getReturnValue()))
						{
							if (dialog.isDontShowAgain()) commonConfiguration.setConfirmation(CommonConfiguration.PARTIAL_EXECUTE, Boolean.TRUE);
						}
						else if ("No".equals(dialog.getReturnValue()))
						{
							if (dialog.isDontShowAgain()) commonConfiguration.setConfirmation(CommonConfiguration.PARTIAL_EXECUTE, Boolean.FALSE);
							queryText=fullText;
						}
						else
							return;
					}
				}

				queryText=queryText.trim();
				executeStatement(queryText, complex);
			}
			else
			{
				ClassLoader classLoader=getQueryClassLoader();
				StatementList statementList=new StatementList(query.getSystemStatement());
				executionThread=new ExecutionThread(statementList, classLoader);
				initExecution(false);
				executionThread.start();
			}
		}
	}

	private void executeStatement(String queryText, boolean complex)
	{
		// The classloader is created here because he has to be created in AWTEventThread
		ClassLoader classLoader=getQueryClassLoader();

		StatementList statementList;
		if (complex) statementList=new StatementList(new ComplexSQLStatement(queryText));
		else statementList=SQLParser.parse(queryText);

		executionThread=new ExecutionThread(statementList, classLoader);
		initExecution(false);
		executionThread.start();
	}

	private ClassLoader getQueryClassLoader()
	{
		ClassLoader classLoader;
		try
		{
			if (CommonConfiguration.getInstance(project).isIncludeProjectClasses())
			{
				classLoader=Utils.getProjectClassLoader(project);
			}
			else
			{
				classLoader=getClass().getClassLoader();
			}
		}
		catch (Throwable e)
		{
			classLoader=getClass().getClassLoader();
		}
		return classLoader;
	}

	public void executeStatement(final SQLStatement statement, final ResultSetPanel panel)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				if (executionThread==null || !executionThread.isAlive())
				{
					pnlStatus.showMessage("");
					errors=false;

					ClassLoader classLoader;
					try
					{
						classLoader=Utils.getProjectClassLoader(project);
					}
					catch (Throwable e)
					{
						classLoader=getClass().getClassLoader();
					}
					executionThread=new ExecutionThread(new StatementList(statement), classLoader);
					executionThread.setResultSetPanel(panel);
					initExecution(true);
					executionThread.start();
				}
			}
		});
	}

	private void initExecution(boolean refresh)
	{
		if (!refresh) clearLastResults();
		executeAction.setEnabled(false);
		suspendAction.setEnabled(true);
		pnlStatus.startProgress();
	}

	private class ExecuteAction extends AbstractAction
	{
		private boolean complex;

		public ExecuteAction(boolean complex)
		{
			super("Query");
			this.complex=complex;
			if (complex)
			{
				putValue(Action.SMALL_ICON, IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.EXECUTE_COMPLEX));
				putValue(Action.SHORT_DESCRIPTION, "Execute complex statements without parsing.");
			}
			else
			{
				putValue(Action.SMALL_ICON, IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.EXECUTE));
				putValue(Action.SHORT_DESCRIPTION, "Execute query.");
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, KeyEvent.CTRL_MASK));
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			try
			{
				executeStatement(complex);
			}
			catch (Exception e1)
			{
				handleThrowable(e1);
			}
		}
	}

	public void handleThrowable(Throwable e)
	{
		e.printStackTrace();
		showMessage(e.getMessage(), ERROR, true, true);
	}

	private void clearLastResults()
	{
		errors=false;
		tfResult.setText("");
		while (pnlResults.getComponentCount()>1) pnlResults.removeTabAt(1);
	}

	public void showMessage(String message, int type, boolean nextLine, boolean activate)
	{
		StyledDocument document=tfResult.getStyledDocument();
		try
		{
			if (document.getLength()>0) document.insertString(document.getLength(), "\n", null);
			switch (type)
			{
				case ERROR:
					document.setLogicalStyle(document.getLength(), document.getStyle("error"));
					break;
				case INFO:
					document.setLogicalStyle(document.getLength(), document.getStyle("info"));
					break;
				default:
					document.setLogicalStyle(document.getLength(), document.getStyle("default"));
			}
			document.insertString(document.getLength(), message, null);
			if (nextLine) document.insertString(document.getLength(), "\n", null);
			tfResult.setCaretPosition(document.getLength());
		}
		catch (BadLocationException e)
		{
			e.printStackTrace();
		}
		if (type==ERROR) errors=true;
		if (activate) pnlResults.setSelectedIndex(0);
	}

	private class CancelAction extends AbstractAction
	{
		public CancelAction()
		{
			super("Close query", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.CANCEL));
			putValue(Action.SHORT_DESCRIPTION, "Close current query.");
		}

		public void actionPerformed(ActionEvent e)
		{
			QueryManager.getInstance(project).removeQuery(query);
		}
	}

	private class StartExportAction extends AbstractAction
	{
		public StartExportAction()
		{
			super("Export", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.EXPORT));
			putValue(Action.SHORT_DESCRIPTION, "Export result table.");
		}

		public void actionPerformed(ActionEvent e)
		{
			JPopupMenu popup=new JPopupMenu("Export");
			popup.add(new JMenuItem(new ExportAction(HTMLExporter.getInstance())));
			popup.add(new JMenuItem(new ExportAction(XMLExporter.getInstance())));
			popup.add(new JMenuItem(new ExportAction(CSVExporter.getInstance())));
			popup.add(new JMenuItem(new ExportAction(FixedWidthExporter.getInstance())));
			popup.add(new JMenuItem(new ExportAction(ExcelExporter.getInstance())));
			popup.addSeparator();
			popup.add(new JMenuItem(new ConfigureExportAction(project)));

			JComponent component=(JComponent)e.getSource();
			popup.show(component, 20, 20);
		}
	}

	private class ExportAction extends AbstractAction
	{
		private Exporter exporter;

		public ExportAction(Exporter exporter)
		{
			super(exporter.getName(),
					IconManager.getIcon(QueryPanel.class.getClassLoader(), "icons/"+exporter.getIcon()+".png"));
			this.exporter=exporter;
			if (exporter.isEnabled() && pnlResults.getSelectedComponent() instanceof ResultSetPanel)
			{
				ResultSetPanel resultSetPanel=(ResultSetPanel)pnlResults.getSelectedComponent();
				setEnabled(resultSetPanel.getResultTable().getModel().getRowCount()>0);
			}
			else
				setEnabled(false);
		}

		public void actionPerformed(ActionEvent e)
		{
			if (pnlResults.getSelectedComponent() instanceof ResultSetPanel)
			{
				ResultSetPanel resultSetPanel=(ResultSetPanel)pnlResults.getSelectedComponent();
				JFileChooser chooser=createExportDialog(exporter.getFileFilter());
				if (JFileChooser.APPROVE_OPTION==chooser.showDialog(null, "Export"))
				{
					File file=chooser.getSelectedFile();
					String exportPath=file.getParent();
					ExportConfiguration exportConfiguration=SQLPluginAppConfig.getInstance().getExport();
					exportConfiguration.setPath(exportPath);
					try
					{
						JTable resultTable=resultSetPanel.getResultTable();
						SQLStatement statement=resultSetPanel.getStatement();
						exporter.exportTable(resultTable, statement, file, exportConfiguration);
					}
					catch (Exception e1)
					{
						e1.printStackTrace();
					}
				}
			}
		}
	}

	private JFileChooser createExportDialog(FileFilter fileFilter)
	{
		JFileChooser chooser=new JFileChooser();
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		chooser.setDialogTitle("Export Result...");
		chooser.addChoosableFileFilter(fileFilter);
		chooser.setFileFilter(fileFilter);

		File path=Utils.getExistingPath(SQLPluginAppConfig.getInstance().getExport().getPath());
		chooser.setCurrentDirectory(path);

		return chooser;
	}

	private class SuspendAction extends AbstractAction
	{
		public SuspendAction()
		{
			super("Suspend", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.SUSPEND));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e)
		{
			if (executionThread!=null && executionThread.isAlive())
			{
				executionThread.cancel();
			}
		}
	}

	private void addTableModel(ResultSetTableModel model, ResultSetPanel panel, boolean select)
	{
		StringBuffer statusText=new StringBuffer();
		if (model.isIncomplete())
		{
			statusText.append(model.getRowCount()+" row(s) loaded.");
			statusText.append(" Result set contained more rows.");
		}
		else
			statusText.append(model.getRowCount()+" row(s) found.");

		if (panel==null)
		{
			int index=pnlResults.getComponentCount();
			ResultSetPanel resultSetPanel=new ResultSetPanel(project, this, model, statusText.toString());
			pnlResults.addTab(String.valueOf(index), IconManager.getIcon(getClass().getClassLoader(), IconNames.RESULT),
					resultSetPanel);
			pnlResults.setToolTipTextAt(pnlResults.getComponentCount()-1, StringUtils.createHTMLToolTip(model.getStatement().getText().trim()));
			if (index==1 || select) pnlResults.setSelectedIndex(index);
		}
		else panel.setTableModel(model);

		showMessage(statusText.toString(), DEFAULT, true, !select);
	}

	public void insertString(final String text)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				Document document=tfQuery.getDocument();
				try
				{
					int selectionStart=tfQuery.getSelectionStart();
					int selectionEnd=tfQuery.getSelectionEnd();
					int position=tfQuery.getCaretPosition();
					if (selectionStart<selectionEnd)
					{
						position=selectionStart;
						document.remove(selectionStart, selectionEnd-selectionStart);
					}
					document.insertString(position, text, null);
					tfQuery.requestFocus();
				}
				catch (BadLocationException e)
				{
					e.printStackTrace();
				}
			}
		});
	}

	public boolean getErrors()
	{
		return errors;
	}

	private class ExecutionThread extends Thread
	{
		private StatementList statementList;
		private ClassLoader classLoader;
		private ResultSetPanel resultSetPanel;
		private PreparedStatement statement;
		private boolean cancelled;

		public ExecutionThread(StatementList statementList, ClassLoader classLoader)
		{
			this.statementList=statementList;
			this.classLoader=classLoader;
		}

		public void run()
		{
			long time1=System.currentTimeMillis();
			try
			{
				Database database=DatabaseManager.getInstance(project).getCurrentDatabase();
				Connection connection=DatabaseUtils.connect(project, database, false);
				if (connection==null) throw new RuntimeException("Connection couldn't be established.");
				Iterator statements=statementList.getStatements().iterator();
				while (statements.hasNext() && !cancelled)
				{
					SQLStatement sqlStatement=(SQLStatement)statements.next();
					if (sqlStatement instanceof DescribeStatement)
						executeDescribe(database, connection, (DescribeStatement)sqlStatement);
					else if (sqlStatement instanceof SystemStatement)
						executeSystem(database, connection, (SystemStatement)sqlStatement);
					else
						executeSelect(database, connection, sqlStatement);
				}
			}
			catch (Throwable e)
			{
				handleThrowable(e);
			}
			finally
			{
				final long time=System.currentTimeMillis()-time1;
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						pnlStatus.appendMessage("Execution time: "+time+"ms");
						if (!errors && pnlResults.getComponentCount()>1) pnlResults.setSelectedIndex(1);
						suspendAction.setEnabled(false);
						executeAction.setEnabled(true);
						pnlStatus.stopProgress();
					}
				});
			}
		}

		private void executeDescribe(Database database, Connection connection, DescribeStatement statement)
				throws Exception
		{
			ResultSet resultSet=null;
			ResultSetTableModel model;
			try
			{
				DatabaseMetaData metaData=connection.getMetaData();
				String catalog=connection.getCatalog();
				String schema=statement.getSchema();
				if (schema==null) schema=database.getDefaultSchema(project).getName();
				String table=statement.getTable().getName();
				if (metaData.storesLowerCaseIdentifiers())
				{
					if (schema!=null) schema=schema.toLowerCase();
					if (table!=null) table=table.toLowerCase();
				}
				else if (metaData.storesUpperCaseIdentifiers())
				{
					if (schema!=null) schema=schema.toUpperCase();
					if (table!=null) table=table.toUpperCase();
				}
				resultSet=metaData.getColumns(catalog, schema, table, null);
				model=new ResultSetTableModel(QueryPanel.this, project, database, statement, resultSet, classLoader);
			}
			finally
			{
				if (resultSet!=null) resultSet.close();
			}
			addTableModel(model, resultSetPanel, false);
		}

		private void executeSelect(Database database, Connection connection, SQLStatement sqlStatement)
				throws Exception
		{
			String text=sqlStatement.getText();
			showMessage(text, INFO, false, true);
			statement=connection.prepareStatement(text);
			try
			{
				boolean hasResultSet=statement.execute();
				int updateCount=statement.getUpdateCount();
				while (hasResultSet || updateCount!=-1)
				{
					if (hasResultSet)
					{
						ResultSetTableModel model;
						ResultSet resultSet=null;
						try
						{
							resultSet=statement.getResultSet();
							model=new ResultSetTableModel(QueryPanel.this, project, database, sqlStatement, resultSet, classLoader);
						}
						finally
						{
							if (resultSet!=null) resultSet.close();
						}
						addTableModel(model, resultSetPanel, false);
					}
					else
						showMessage(sqlStatement.getSuccessMessage(updateCount), DEFAULT, true, true);
					hasResultSet=statement.getMoreResults();
					updateCount=-1;
				}
			}
			finally
			{
				if (statement!=null) statement.close();
				statement=null;
			}
		}

		private void executeSystem(Database database, Connection connection, SystemStatement sqlStatement)
				throws Exception
		{
			String text=sqlStatement.getText();
			showMessage(text, INFO, false, true);
			statement=connection.prepareStatement(text);
			try
			{
				List parameters=sqlStatement.getParameters();
				for (int i=0; i<parameters.size(); i++) statement.setObject(i+1, parameters.get(i));
				ResultSet resultSet=statement.executeQuery();
				ResultSetTableModel model=new ResultSetTableModel(QueryPanel.this, project, database, sqlStatement, resultSet, classLoader);
				addTableModel(model, resultSetPanel, true);
			}
			finally
			{
				if (statement!=null) statement.close();
				statement=null;
			}
		}

		public void cancel()
		{
			cancelled=true;
			PreparedStatement statement=this.statement;
			if (statement!=null)
			{
				try
				{
					statement.cancel();
					showMessage("Query aborted by user.", ERROR, true, true);
				}
				catch (SQLException e)
				{
					handleThrowable(e);
				}
			}
		}

		public void setResultSetPanel(ResultSetPanel resultSetPanel)
		{
			this.resultSetPanel=resultSetPanel;
		}
	}

	private class ConfigurationListener implements Observer
	{
		public void update(Observable o, Object arg)
		{
			NotifyObject note=(NotifyObject)arg;
			if ("defaultFormat changed".equals(note.getArgument(0)))
			{
				pnlResults.repaint();
			}
		}
	}

	private class AutoCommitAction extends AbstractAction
	{
		public AutoCommitAction()
		{
			super("Auto-Commit:");
			putValue(Action.SHORT_DESCRIPTION, "The 'Auto-Commit' value for the current database.");
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e)
		{
			Database currentDatabase=DatabaseManager.getInstance(project).getCurrentDatabase();
			if (currentDatabase!=null)
			{
				Object value=currentDatabase.getProperty(DatabaseDriver.AUTO_COMMIT);
				if (Boolean.TRUE.equals(value))
					currentDatabase.setProperty(DatabaseDriver.AUTO_COMMIT, Boolean.FALSE);
				else if (Boolean.FALSE.equals(value))
					currentDatabase.setProperty(DatabaseDriver.AUTO_COMMIT, null);
				else
					currentDatabase.setProperty(DatabaseDriver.AUTO_COMMIT, Boolean.TRUE);
			}
		}

		public void update(Database currentDatabase)
		{
			boolean enabled=currentDatabase!=null;
			setEnabled(enabled);
			if (enabled)
			{
				Object value=currentDatabase.getProperty(DatabaseDriver.AUTO_COMMIT);
				if (Boolean.TRUE.equals(value))
					putValue(Action.NAME, "Auto-Commit: ON");
				else if (Boolean.FALSE.equals(value))
					putValue(Action.NAME, "Auto-Commit: OFF");
				else
					putValue(Action.NAME, "Auto-Commit: DEFAULT");
			}
		}
	}

	private class TextPaneMouseListener extends MouseAdapter
	{
		public void mousePressed(MouseEvent e)
		{
			if (e.isPopupTrigger()) showPopup(e);
		}

		public void mouseReleased(MouseEvent e)
		{
			if (e.isPopupTrigger()) showPopup(e);
		}

		private void showPopup(MouseEvent e)
		{
			JPopupMenu popup=new JPopupMenu();
			popup.add(new SaveTemplateAction());
			popup.show(e.getComponent(), e.getX(), e.getY());
		}
	}

	private class SaveTemplateAction extends AbstractAction
	{
		public SaveTemplateAction()
		{
			super("Save as Template...", IconManager.ICON_16X16);
			validate();
		}

		public void validate()
		{
			String text=tfQuery.getSelectedText();
			if (text==null) text=tfQuery.getText();
			setEnabled(!StringUtils.isEmpty(text));
		}

		public void actionPerformed(ActionEvent e)
		{
			String text=tfQuery.getSelectedText();
			if (text==null) text=tfQuery.getText();
			if (!StringUtils.isEmpty(text))
			{
				EditTemplateDialogStub dialog=new EditTemplateDialogStub(TemplateManager.getInstance(), null, text);
				Utils.wrapDialog(project, dialog, true, true);
			}
		}
	}
}
