/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.*;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;

import com.kiwisoft.db.Database;
import com.kiwisoft.db.DatabaseManager;
import com.kiwisoft.db.Query;
import com.kiwisoft.db.QueryManager;
import com.kiwisoft.db.driver.DatabaseDriver;
import com.kiwisoft.sqlPlugin.actions.AboutAction;
import com.kiwisoft.sqlPlugin.actions.HelpAction;
import com.kiwisoft.sqlPlugin.actions.PropertiesAction;
import com.kiwisoft.sqlPlugin.browser.BrowserPanel;
import com.kiwisoft.sqlPlugin.config.SQLPluginAppConfig;
import com.kiwisoft.sqlPlugin.templates.StatementTemplate;
import com.kiwisoft.sqlPlugin.templates.StatementTemplateFormat;
import com.kiwisoft.utils.NotifyObject;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.ListenerSupport;
import com.kiwisoft.utils.format.ObjectFormat;
import com.kiwisoft.utils.format.ByteArrayFormat;
import com.kiwisoft.utils.gui.ConfirmationDialogStub;
import com.kiwisoft.utils.gui.ExtentionFileFilter;
import com.kiwisoft.utils.gui.IconManager;
import com.kiwisoft.utils.gui.table.RendererFactory;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.7 $, $Date: 2005/08/10 18:54:14 $
 */
public class SQLPluginPanel extends JPanel
{
	private static Map instanceMap=new WeakHashMap();

	public static SQLPluginPanel getInstance(Project project, boolean create)
	{
		SQLPluginPanel instance=(SQLPluginPanel)instanceMap.get(project);
		if (instance==null && create)
		{
			try
			{
				instance=new SQLPluginPanel(project);
				instanceMap.put(project, instance);
			}
			catch (Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		return instance;
	}

	public static void closeInstance(Project project)
	{
		SQLPluginPanel instance=(SQLPluginPanel)instanceMap.get(project);
		if (instance!=null) instance.dispose();
		instanceMap.remove(project);
	}

	private JTabbedPane tabs;
	private JComboBox cbxDatabases;
	private JComboBox cbxGroups;
	private Project project;

	private CopyQueryAction copyQueryAction;
	private SaveAction saveAction;
	private CommitAction commitAction;
	private RollbackAction rollbackAction;

	private ListenerSupport listenerSupport=new ListenerSupport();

	private SQLPluginPanel(Project project)
	{
		RendererFactory rendererFactory=RendererFactory.getInstance();
		rendererFactory.setRenderer(StatementTemplate.class, new StatementTemplateFormat(ObjectFormat.DEFAULT));
		rendererFactory.setRenderer(byte[].class, new ByteArrayFormat(ObjectFormat.DEFAULT));

		this.project=project;
		setLayout(new BorderLayout());

		DatabaseManager databaseManager=DatabaseManager.getInstance(project);

		CurrentDatabaseListener currentDatabaseListener=new CurrentDatabaseListener();

		List groups=new LinkedList(databaseManager.getGroups());
		Collections.sort(groups);
		groups.add(0, DatabaseManager.ALL_GROUPS);
		cbxGroups=new JComboBox(groups.toArray());
		cbxGroups.setActionCommand("group");
		listenerSupport.installActionListener(cbxGroups, currentDatabaseListener);
		cbxGroups.setFocusable(false);

		String currentGroup=databaseManager.getCurrentGroup();
		Database currentDatabase=databaseManager.getCurrentDatabase();
		List databases=new LinkedList(databaseManager.getDatabases(currentGroup));
		databases.add(DatabaseManager.NO_DATABASE);

		cbxDatabases=new JComboBox(databases.toArray());
		cbxDatabases.setActionCommand("database");
		listenerSupport.installActionListener(cbxDatabases, currentDatabaseListener);
		cbxDatabases.setFocusable(false);

		if (currentGroup!=null)
			cbxGroups.setSelectedItem(currentGroup);
		else
			cbxGroups.setSelectedItem(DatabaseManager.ALL_GROUPS);

		if (currentDatabase!=null)
		{
			cbxDatabases.setSelectedItem(currentDatabase);
			cbxDatabases.setToolTipText(getToolTip(currentDatabase));
		}
		else
			cbxDatabases.setSelectedItem(DatabaseManager.NO_DATABASE);

		JToolBar toolbar=new JToolBar();
		toolbar.setFloatable(false);
		toolbar.add(Utils.createButton(new OpenAction()));
		saveAction=new SaveAction();
		toolbar.add(Utils.createButton(saveAction));
		toolbar.addSeparator();
		toolbar.add(Utils.createButton(new CreateQueryAction()));
		copyQueryAction=new CopyQueryAction();
		toolbar.add(Utils.createButton(copyQueryAction));
		toolbar.addSeparator();
		toolbar.add(cbxDatabases);
		toolbar.add(cbxGroups);
		toolbar.add(Utils.createButton(new PropertiesAction(project)));
		toolbar.add(Utils.createButton(new BrowserAction()));
		toolbar.add(Utils.createButton(new DisconnectAction()));
		commitAction=new CommitAction();
		toolbar.add(Utils.createButton(commitAction));
		rollbackAction=new RollbackAction();
		toolbar.add(Utils.createButton(rollbackAction));
		toolbar.addSeparator();
		toolbar.add(Utils.createButton(new HelpAction()));
		toolbar.add(Utils.createButton(new AboutAction(this)));

		tabs=new JTabbedPane();
		tabs.setTabPlacement(JTabbedPane.BOTTOM);
		listenerSupport.installChangeListener(tabs, new QuerySelectionListener());

		add(toolbar, BorderLayout.NORTH);
		add(tabs, BorderLayout.CENTER);

		QueryManager queryList=QueryManager.getInstance(project);
		listenerSupport.installObserver(queryList, new QueryListObserver());
		listenerSupport.installObserver(databaseManager, new DatabaseManagerObserver());

		List queries=queryList.getQueries();
		if (queries.isEmpty())
			queryList.createQuery();
		else
		{
			Iterator it=queries.iterator();
			while (it.hasNext()) addQuery((Query)it.next());
		}
		updateActions();

		listenerSupport.installMouseListener(tabs, new TabsMouseListener());
	}

	private Query getSelectedQuery()
	{
		if (tabs!=null)
		{
			QueryPanel queryPanel=(QueryPanel)tabs.getSelectedComponent();
			if (queryPanel!=null) return queryPanel.getQuery();
		}
		return null;
	}

	private String getToolTip(Database database)
	{
		StringBuffer toolTip=new StringBuffer();
		toolTip.append("<html>");
		toolTip.append(database.getURL());
		DatabaseDriver driver=database.getDatabaseDriver();
		if (driver!=null)
		{
			Properties properties=driver.getConnectProperties(database);
			driver.addCustomProperties(database, properties);
			toolTip.append("<small>");
			for (Iterator it=properties.entrySet().iterator(); it.hasNext();)
			{
				Map.Entry entry=(Map.Entry)it.next();
				toolTip.append("<br>&nbsp;&nbsp;");
				toolTip.append(entry.getKey());
				toolTip.append("=");
				toolTip.append(entry.getValue());
			}
			toolTip.append("</small>");
		}
		toolTip.append("</html>");
		return toolTip.toString();
	}

	private QueryPanel getQueryPanel(Query query)
	{
		Component[] panels=tabs.getComponents();
		for (int i=0; i<panels.length; i++)
		{
			if (panels[i] instanceof QueryPanel)
			{
				QueryPanel queryPanel=(QueryPanel)panels[i];
				if (queryPanel.getQuery()==query) return queryPanel;
			}
		}
		return null;
	}

	private void addQuery(Query newQuery)
	{
		final QueryPanel queryPanel=new QueryPanel(project, newQuery);
		tabs.add(getTabName(newQuery), queryPanel);
		tabs.setSelectedComponent(queryPanel);
		if (newQuery.isSystemQuery())
		{
			SwingUtilities.invokeLater(new Runnable()
			{
				public void run()
				{
					queryPanel.executeStatement(false);
				}
			});
		}
	}

	public void createQuery(final String text, final boolean execute)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				Query query=QueryManager.getInstance(project).createQuery(text);
				if (execute)
				{
					QueryPanel queryPanel=getActiveQueryPanel();
					if (queryPanel.getQuery()==query) queryPanel.executeStatement(false);
				}
			}
		});
	}

	private class QuerySelectionListener implements ChangeListener
	{
		public void stateChanged(ChangeEvent e)
		{
			updateActions();
		}
	}

	private static String getTabName(Query query)
	{
		return query.getName()+(query.isSaveable() ? "*" : "");
	}

	private class QueryListObserver implements Observer
	{
		public void update(Observable o, Object arg)
		{
			NotifyObject note=(NotifyObject)arg;
			Object type=note.getArgument(0);
			if ("query added".equals(type))
			{
				Query newQuery=(Query)note.getArgument(1);
				addQuery(newQuery);
				updateActions();
			}
			else if ("query removed".equals(type))
			{
				Query query=(Query)note.getArgument(1);
				QueryPanel queryPanel=getQueryPanel(query);
				if (queryPanel!=null) tabs.remove(queryPanel);
				updateActions();
				if (tabs.getTabCount()==0 && project!=null)
				{
					ToolWindow toolWindow=ToolWindowManager.getInstance(project).getToolWindow(SQLQueryPlugin.SQL_TOOL_WINDOW);
					if (toolWindow!=null) toolWindow.hide(null);
				}
			}
			else if ("query changed".equals(type) || "query saved".equals(type) || "query name changed".equals(type))
			{
				Query query=(Query)note.getArgument(1);
				QueryPanel queryPanel=getQueryPanel(query);
				if (queryPanel!=null)
				{
					int index=tabs.indexOfComponent(queryPanel);
					tabs.setTitleAt(index, getTabName(query));
				}
				updateActions();
			}
		}
	}

	private void updateActions()
	{
		Query selectedQuery=getSelectedQuery();
		boolean querySelected=selectedQuery!=null;
		boolean databaseSelected=DatabaseManager.getInstance(project).getCurrentDatabase()!=null;
		if (copyQueryAction!=null) copyQueryAction.setEnabled(querySelected);
		if (saveAction!=null) saveAction.setEnabled(querySelected && selectedQuery.isSaveable());
		if (commitAction!=null) commitAction.setEnabled(databaseSelected);
		if (rollbackAction!=null) rollbackAction.setEnabled(databaseSelected);
	}

	private class CurrentDatabaseListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			if ("database".equals(e.getActionCommand()))
			{
				Object selectedItem=cbxDatabases.getSelectedItem();
				if (selectedItem instanceof Database)
					DatabaseManager.getInstance(project).setCurrentDatabase((Database)selectedItem);
				else
					DatabaseManager.getInstance(project).setCurrentDatabase(null);
			}
			else if ("group".equals(e.getActionCommand()))
			{
				Object selectedItem=cbxGroups.getSelectedItem();
				if (selectedItem instanceof String)
					DatabaseManager.getInstance(project).setCurrentGroup((String)selectedItem);
				else
					DatabaseManager.getInstance(project).setCurrentGroup(null);
			}
		}
	}

	private class DatabaseManagerObserver implements Observer
	{
		public void update(Observable o, Object arg)
		{
			NotifyObject note=(NotifyObject)arg;
			DatabaseManager databaseManager=DatabaseManager.getInstance(project);
			if ("databases changed".equals(note.getArgument(0)))
			{
				List groups=new LinkedList(databaseManager.getGroups());
				Collections.sort(groups);
				groups.add(0, DatabaseManager.ALL_GROUPS);
				cbxGroups.setModel(new DefaultComboBoxModel(groups.toArray()));

				String currentGroup=databaseManager.getCurrentGroup();
				Database currentDatabase=databaseManager.getCurrentDatabase();
				List databases=new LinkedList(databaseManager.getDatabases(currentGroup));
				databases.add(DatabaseManager.NO_DATABASE);
				cbxDatabases.setModel(new DefaultComboBoxModel(databases.toArray()));

				if (currentGroup!=null)
					cbxGroups.setSelectedItem(currentGroup);
				else
					cbxGroups.setSelectedItem(DatabaseManager.ALL_GROUPS);
				if (currentDatabase!=null)
				{
					cbxDatabases.setSelectedItem(currentDatabase);
					cbxDatabases.setToolTipText(getToolTip(currentDatabase));
				}
				else
				{
					cbxDatabases.setSelectedItem(DatabaseManager.NO_DATABASE);
					cbxDatabases.setToolTipText(null);
				}
			}
			else if ("current database changed".equals(note.getArgument(0)))
			{
				Object selected=cbxDatabases.getSelectedItem();
				Object currentDatabase=databaseManager.getCurrentDatabase();
				if (currentDatabase==null) currentDatabase=DatabaseManager.NO_DATABASE;
				if (selected!=currentDatabase) cbxDatabases.setSelectedItem(currentDatabase);
				if (currentDatabase instanceof Database)
					cbxDatabases.setToolTipText(getToolTip(((Database)currentDatabase)));
				else
					cbxDatabases.setToolTipText(null);
			}
			else if ("current group changed".equals(note.getArgument(0)))
			{
				Object selected=cbxGroups.getSelectedItem();
				Object currentGroup=databaseManager.getCurrentGroup();
				if (currentGroup==null) currentGroup=DatabaseManager.NO_DATABASE;
				if (selected!=currentGroup) cbxGroups.setSelectedItem(currentGroup);

				Database currentDatabase=databaseManager.getCurrentDatabase();
				List databases=new LinkedList(databaseManager.getDatabases(databaseManager.getCurrentGroup()));
				databases.add(DatabaseManager.NO_DATABASE);
				cbxDatabases.setModel(new DefaultComboBoxModel(databases.toArray()));
				if (currentDatabase!=null)
				{
					cbxDatabases.setSelectedItem(currentDatabase);
					cbxDatabases.setToolTipText(getToolTip(currentDatabase));
				}
				else
				{
					cbxDatabases.setSelectedItem(DatabaseManager.NO_DATABASE);
					cbxDatabases.setToolTipText(null);
				}
			}
			updateActions();
		}
	}

	public void dispose()
	{
		listenerSupport.dispose();
	}

	public void removeNotify()
	{
		if (!SQLPluginAppConfig.getInstance().isKeepConnectionOpen())
			DatabaseManager.getInstance(project).closeAllConnections();
		super.removeNotify();
	}

	public QueryPanel getActiveQueryPanel()
	{
		Component selectedComponent=tabs.getSelectedComponent();
		if (selectedComponent instanceof QueryPanel) return (QueryPanel)selectedComponent;
		return null;
	}

	private class CreateQueryAction extends AbstractAction
	{
		public CreateQueryAction()
		{
			super(null, IconManager.getIcon(SQLPluginPanel.class.getClassLoader(), IconNames.ADD));
			putValue(Action.SHORT_DESCRIPTION, "Open a new empty query.");
		}

		public void actionPerformed(ActionEvent e)
		{
			QueryManager.getInstance(project).createQuery();
		}
	}

	private class CopyQueryAction extends AbstractAction
	{
		public CopyQueryAction()
		{
			super(null, IconManager.getIcon(SQLPluginPanel.class.getClassLoader(), IconNames.COPY));
			putValue(Action.SHORT_DESCRIPTION, "Create a new query and copy the statement from the selected.");
		}

		public void actionPerformed(ActionEvent e)
		{
			QueryPanel queryPanel=(QueryPanel)tabs.getSelectedComponent();
			if (queryPanel!=null)
			{
				Query query=queryPanel.getQuery();
				if (query!=null)
				{
					QueryManager.getInstance(project).createQuery(query, queryPanel.getQueryText());
				}
			}
		}
	}

	private class OpenAction extends AbstractAction
	{
		public OpenAction()
		{
			super("Open", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.OPEN));
		}

		public void actionPerformed(ActionEvent e)
		{
			JFileChooser chooser=new JFileChooser();
			chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			chooser.setDialogType(JFileChooser.OPEN_DIALOG);
			chooser.setDialogTitle("Load Query");
			chooser.setMultiSelectionEnabled(false);
			chooser.setCurrentDirectory(Utils.getExistingPath(CommonConfiguration.getInstance(project).getQueryPath()));

			ExtentionFileFilter sqlFileFilter=new ExtentionFileFilter("SQL Sources", new String[]{"sql", "ddl"});
			ExtentionFileFilter txtFileFilter=new ExtentionFileFilter("Text Files", "txt");
			chooser.addChoosableFileFilter(sqlFileFilter);
			chooser.addChoosableFileFilter(txtFileFilter);
			chooser.setFileFilter(sqlFileFilter);

			int res=chooser.showDialog(SQLPluginPanel.this, "Load");
			if (res==JFileChooser.APPROVE_OPTION)
			{
				File file=chooser.getSelectedFile();
				CommonConfiguration.getInstance(project).setQueryPath(file.getParent());
				if (file.canRead())
				{
					FileInputStream fis=null;
					try
					{
						fis=new FileInputStream(file);
						byte[] buf=new byte[fis.available()];
						fis.read(buf);
						QueryManager.getInstance(project).createQuery(file, new String(buf));
					}
					catch (IOException e1)
					{
						JOptionPane.showMessageDialog(SQLPluginPanel.this, e1.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					}
					finally
					{
						if (fis!=null)
						{
							try
							{
								fis.close();
							}
							catch (IOException e1)
							{
								JOptionPane.showMessageDialog(SQLPluginPanel.this, e1.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
						}
					}
				}
				else
				{
					JOptionPane.showMessageDialog(SQLPluginPanel.this, "Can't read file '"+file+"'", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	private static File addDefaultExtension(File file, String extension)
	{
		File parent=file.getParentFile();
		String fileName=file.getName();
		int index=fileName.lastIndexOf('.');
		if (index<0)
			return new File(parent, fileName+'.'+extension);
		else
			return file;
	}

	private class SaveAction extends AbstractAction
	{
		public SaveAction()
		{
			super("Save", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.SAVE));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e)
		{
			Query selectedQuery=getSelectedQuery();
			if (selectedQuery!=null)
			{
				JFileChooser chooser=new JFileChooser();
				chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				chooser.setDialogType(JFileChooser.SAVE_DIALOG);
				chooser.setDialogTitle("Save Query as ...");
				chooser.setMultiSelectionEnabled(false);
				if (selectedQuery.getFile()!=null)
					chooser.setSelectedFile(selectedQuery.getFile());
				else
					chooser.setCurrentDirectory(Utils.getExistingPath(CommonConfiguration.getInstance(project).getQueryPath()));

				ExtentionFileFilter sqlFileFilter=new ExtentionFileFilter("SQL Sources", new String[]{"sql", "ddl", "pgb", "pgh"});
				ExtentionFileFilter txtFileFilter=new ExtentionFileFilter("Text Files", "txt");

				sqlFileFilter.setShowReadOnly(false);
				txtFileFilter.setShowReadOnly(false);

				chooser.addChoosableFileFilter(sqlFileFilter);
				chooser.addChoosableFileFilter(txtFileFilter);
				chooser.setFileFilter(sqlFileFilter);

				int res=chooser.showDialog(SQLPluginPanel.this, "Save As");
				if (res==JFileChooser.APPROVE_OPTION)
				{
					File file=chooser.getSelectedFile();
					file=addDefaultExtension(file, "sql");
					CommonConfiguration.getInstance(project).setQueryPath(file.getParent());
					try
					{
						if (file.exists())
						{
							ConfirmationDialogStub dialog=new ConfirmationDialogStub("Warning", "File '"+file.getName()+"' exists. Do you want override the file ?",
									new String[]{"Yes", "No"}, "No", false);
							Utils.wrapDialog(project, dialog, true, true);
							if ("Yes".equals(dialog.getReturnValue())) save(file, selectedQuery);
						}
						else
						{
							if (file.createNewFile()) save(file, selectedQuery);
						}
					}
					catch (Exception e1)
					{
						JOptionPane.showMessageDialog(SQLPluginPanel.this, e1.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		}

		private void save(File file, Query query) throws Exception
		{
			FileWriter fw=null;
			try
			{
				fw=new FileWriter(file);
				fw.write(query.getStatement());
				query.setFile(file);
				query.setChanged(false);
			}
			finally
			{
				if (fw!=null)
				{
					fw.close();
				}
			}
		}
	}

	private class BrowserAction extends AbstractAction
	{
		public BrowserAction()
		{
			super("Browser", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.BROWSER));
			putValue(Action.SHORT_DESCRIPTION, "Show database struture.");
		}

		public void actionPerformed(ActionEvent e)
		{
			if (project!=null)
			{
				ToolWindowManager manager=ToolWindowManager.getInstance(project);
				BrowserPanel browserPanel=BrowserPanel.getInstance(project);
				if (browserPanel!=null)
				{
					ToolWindow console=manager.getToolWindow(SQLQueryPlugin.BROWSER_TOOL_WINDOW);
					if (console==null)
					{
						console=manager.registerToolWindow(SQLQueryPlugin.BROWSER_TOOL_WINDOW, browserPanel, ToolWindowAnchor.BOTTOM);
						if (console!=null)
						{
							console.setIcon(IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.DATABASE_STRUCTURE));
						}
					}
					if (console!=null)
					{
						console.show(null);
						console.activate(null);
					}
				}
			}
			else
			{
				JFrame frame=new JFrame("SQL Schema");
				frame.getContentPane().add(new BrowserPanel(null));
				frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
				frame.setIconImage(IconManager.getIcon(SQLPluginPanel.class.getClassLoader(), IconNames.DATABASE_STRUCTURE).getImage());
				frame.setSize(800, 300);
				frame.setVisible(true);
			}
		}
	}

	private class DisconnectAction extends AbstractAction
	{
		public DisconnectAction()
		{
			super("Disconnect", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.DISCONNECT));
			putValue(Action.SHORT_DESCRIPTION, "Close all connections.");
		}

		public void actionPerformed(ActionEvent e)
		{
			CommonConfiguration configuration=CommonConfiguration.getInstance(project);
			Boolean confirm=configuration.getConfirmation(CommonConfiguration.CONFIRM_DISCONNECT);
			if (Boolean.TRUE.equals(confirm))
				DatabaseManager.getInstance(project).closeAllConnections();
			else
			{
				ConfirmationDialogStub dialog=new ConfirmationDialogStub("Question", "Do you really want to close all connections?",
						new String[]{"Yes", "No"}, "No", true);
				Utils.wrapDialog(project, dialog, true, true);
				if ("Yes".equals(dialog.getReturnValue()))
				{
					if (dialog.isDontShowAgain()) configuration.setConfirmation(CommonConfiguration.CONFIRM_DISCONNECT, Boolean.TRUE);
					DatabaseManager.getInstance(project).closeAllConnections();
				}
			}
		}
	}

	private class CommitAction extends AbstractAction
	{
		public CommitAction()
		{
			super("Commit", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.COMMIT));
			putValue(Action.SHORT_DESCRIPTION, "Commit all changes.");
			setEnabled(DatabaseManager.getInstance(project).getCurrentDatabase()!=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			Database database=DatabaseManager.getInstance(project).getCurrentDatabase();
			if (database!=null)
			{
				Connection connection=database.getConnection();
				try
				{
					if (connection!=null)
					{
						CommonConfiguration configuration=CommonConfiguration.getInstance(project);
						Boolean confirm=configuration.getConfirmation(CommonConfiguration.CONFIRM_COMMIT);
						if (Boolean.TRUE.equals(confirm))
							connection.commit();
						else
						{
							ConfirmationDialogStub dialog=new ConfirmationDialogStub("Question", "Do you really want to commit the connection?",
									new String[]{"Yes", "No"}, "No", true);
							Utils.wrapDialog(project, dialog, true, true);
							if ("Yes".equals(dialog.getReturnValue()))
							{
								if (dialog.isDontShowAgain()) configuration.setConfirmation(CommonConfiguration.CONFIRM_COMMIT, Boolean.TRUE);
								connection.commit();
							}
						}
					}
					else
					{
						JOptionPane.showMessageDialog(SQLPluginPanel.this, "No connection open.", "Information", JOptionPane.INFORMATION_MESSAGE);
					}
				}
				catch (SQLException e1)
				{
					e1.printStackTrace();
					JOptionPane.showMessageDialog(SQLPluginPanel.this, e1.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	private class RollbackAction extends AbstractAction
	{
		public RollbackAction()
		{
			super("Rollback", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.ROLLBACK));
			putValue(Action.SHORT_DESCRIPTION, "Rollback all changes.");
			setEnabled(DatabaseManager.getInstance(project).getCurrentDatabase()!=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			Database database=DatabaseManager.getInstance(project).getCurrentDatabase();
			if (database!=null)
			{
				Connection connection=database.getConnection();
				try
				{
					if (connection!=null)
					{
						CommonConfiguration configuration=CommonConfiguration.getInstance(project);
						Boolean confirm=configuration.getConfirmation(CommonConfiguration.CONFIRM_ROLLBACK);
						if (Boolean.TRUE.equals(confirm))
							connection.rollback();
						else
						{
							ConfirmationDialogStub dialog=new ConfirmationDialogStub("Question", "Do you really want to rollback the connection?",
									new String[]{"Yes", "No"}, "No", true);
							Utils.wrapDialog(project, dialog, true, true);
							if ("Yes".equals(dialog.getReturnValue()))
							{
								if (dialog.isDontShowAgain()) configuration.setConfirmation(CommonConfiguration.CONFIRM_ROLLBACK, Boolean.TRUE);
								connection.rollback();
							}
						}
					}
					else
					{
						JOptionPane.showMessageDialog(SQLPluginPanel.this, "No connection open.", "Information", JOptionPane.INFORMATION_MESSAGE);
					}
				}
				catch (SQLException e1)
				{
					e1.printStackTrace();
					JOptionPane.showMessageDialog(SQLPluginPanel.this, e1.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	private class TabsMouseListener extends MouseAdapter
	{
		public void mousePressed(MouseEvent e)
		{
			if ((e.getModifiers()&MouseEvent.BUTTON3_MASK)!=0)
			{
				int index=tabs.getUI().tabForCoordinate(tabs, e.getX(), e.getY());
				if (index>=0)
				{
					Component component=tabs.getComponentAt(index);
					if (component instanceof QueryPanel)
					{
						QueryPanel panel=(QueryPanel)component;
						JPopupMenu popup=new JPopupMenu();
						popup.add(new JMenuItem(new RenameQueryAction(panel.getQuery())));
						popup.show(tabs, e.getX(), e.getY()-20);
					}
				}
			}
		}
	}

	private class RenameQueryAction extends AbstractAction
	{
		private Query query;

		public RenameQueryAction(Query query)
		{
			super("Rename query");
			this.query=query;
			setEnabled(query!=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			RenameDialogStub dialogStub=new RenameDialogStub("Rename Query", query.getName());
			Utils.wrapDialog(project, dialogStub, true, true);
			String newName=dialogStub.getReturnValue();
			if (!StringUtils.isEmpty(newName)) query.setName(newName);
		}
	}
}
