/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.utils.format;

import javax.swing.Icon;

/**
 * Interface class for displaying objects in UI text components like table cells, lists,
 * labels and text fields.
 *
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2005/08/10 18:55:15 $
 */
public interface ObjectFormat
{
	/**
	 * Used for default formats.
	 * @see #getName()
	 */
	public static final String DEFAULT="Default";

	/**
	 * Returns the name of the format. Return "Default" if the format should be the default format
	 * for the handled classes (works only if no default format has already been installed).
	 * @return The name of the format. Must not be <code>null</code>.
	 */
	String getName();

	/**
	 * The name of the group the format belongs to. The group defines the sub menu under which
	 * the format will appear in the context menu.
	 *
	 * @return The name of the format group. If <code>null</code> the format will appear in the root menu.
	 */
	String getGroup();

	/**
	 * @return <code>true</code> if the format can format objects of the specified class.
	 * @see #format(Object)
	 */
	boolean canFormat(Class aClass);

	/**
	 * @return <code>true</code> if the format can create objects of the specified class
	 * from <code>String</code> objects.
	 * @see #parse(String, Class)
	 */
	boolean canParse(Class aClass);

	/**
	 * Returns the horizontal alignment of the formatted text.
	 * @param value The unformated value.
	 * @return One of the following constants
	 *           defined in <code>SwingConstants</code>:
	 *           <code>LEFT</code>,
	 *           <code>CENTER</code>,
	 *           <code>RIGHT</code>,
	 *           <code>LEADING</code>.
	 *           <code>TRAILING</code>.
	 */
	int getHorizontalAlignment(Object value);

	/**
	 * Returns the icon for the specified value;
	 * @param value The unformated value.
	 * @return The icon for the specified value (can be null).
	 */
	Icon getIcon(Object value);

	/**
	 * Returns the <code>String</code> representing the specified value.
	 * @param value The object which should be formated.
	 * @return The string representing the specified object.
	 * @see #canFormat(Class)
	 */
	String format(Object value);

	/**
	 * Returns the <code>Object</code> created from the specified <code>String</code>.
	 * The method may throw an <code>RuntimeException</code> if the <code>String</code> is invalid.
	 * @param value The <code>String</code> which should be parsed.
	 * @param targetClass The <code>Class</code> the return value should have. Can be ignored if
	 * the format handles only one class.
	 * @return The <code>Object</code> created from the <code>String</code>.
	 * @see #canParse(Class)
	 */
	Object parse(String value, Class targetClass);
}
